<?
// php-Script zur Anzeige von .crid-Aufnahmen von Siemens M740AV/M750S-Receivern
// Modifizert von Stefan Onderka (stefan@onderka.com) - Lizenz: GPL
// Version 1.6 2007-10-31.Helloween, http://www.onderka.com
//
// Basiert auf "M740-Fernsteuerung", herausgegeben 2005-11-12 von "api" 
// (m740-api@users.sourceforge.net)
//
// Links:
//
//   http://m740.de/wiki/M740-Fernsteuerung
//   http://prdownloads.sourceforge.net/waveboxremote/2006-11-12-api-m740-fernsteuerung.zip?download

// Konfiguration #############################################################
include('./config.php');

// System-Info
include('./system.php');

// .crid-Timestamps sind in GMT.
// GMT Setzen, um Sendungslaengen korrekt zu berechnen
date_default_timezone_set("GMT+0");

// Ist eine Box online?
function box_alive($box_ip) {
   global $box_alive_port;
   global $box_alive_0;
   global $box_alive_1;
   $socket=@fsockopen($box_ip,$box_alive_port);
   if ($socket != false) {
      // Reagiert
      fclose($socket);
      return true;
   } else {
      // Reagiert nicht
      return false;
      }
   } 

// Aufnahmegroesse zur .crid-Datei zusammenzaehlen
function record_file_size($file) {
   // Groesse zuruecksetzen
   $set_filesize = 0;
   $set_filecount = 0;
   if ($set_handle = opendir(dirname($file))) {
      // Verzeichnis durchlaufen
      while (false !== ($set_file = readdir($set_handle))) {
         // Wenn Datei den Namen der .crid-Datei enthaelt (x.crid.mpg), dazuzaehlen
         if (strstr(basename($set_file), basename($file))) {
            //echo "Teil-Datei: ".$set_file." (".filesize(dirname($file)."/".$set_file)." Byte)<br>";
            $set_filesize = $set_filesize + filesize(dirname($file)."/".$set_file);
            $set_filecount +=1;
            }
      }
      // Array mit [0]=Bytes, [1]=Dateizahl
      return array($set_filesize, $set_filecount);
      }
   }

// Sonderzeichen loeschen/ersetzen
function clean_str($str) {
   $str = str_replace(chr(0x8a), "\n", $str);
   $str = str_replace(chr(0x05), "", $str);
   $str = trim($str, "\x00..\x20");
   return $str;
   }

// ":" und "-" aus MAC-Adressen schneiden, Grossbuchstaben
function convert_mac($str) {
   $str = str_replace(":", "", $str);
   $str = str_replace("-", "", $str);
   $str = strtoupper($str);
   return $str;
   }

// Sperr-Nummer zu Text
function sperr_nr_to_text($num) {
   global $string_locked_0;
   global $string_locked_1;

   switch($num) {
   case 1;
      // gesperrt
      return $string_locked_1;
      break;
   case 0;
      // nicht gesperrt
      return $string_locked_0;
      break;
      }
   }

// Status zu Text
function status_nr_to_text($num) {
   global $string_progress_0;
   global $string_progress_1;
   global $string_progress_2;
   global $string_progress_3;
   global $string_progress_4;
   global $string_progress_5;
   global $string_progress_6;
   global $string_progress_9;

   switch($num) {
   case 1;
      return $string_progress_1;
      break;
   case 2;
      return $string_progress_2;
      break;
   case 3;
      return $string_progress_3;
      break;
   case 4;
      return $string_progress_4;
      break;
   case 5;
      return $string_progress_5;
      break;
   case 6;
      return $string_progress_6;
      break;
   case 9;
      return $string_progress_9;
      break;
   default:
      return $string_progress_0;
      }
   }

// EPGF zu Text
function epgf_nr_to_text($num) {
   global $string_record_type_1;
   global $string_record_type_2;
   global $string_record_type_4;
   global $string_record_type_8;
   global $string_record_type_32;

   switch($num) {
   case 1;
      return $string_record_type_1;
      break;
   case 2;
      return $string_record_type_2;
      break;
   case 4;
      return $string_record_type_4;
      break;
   case 8;
      return $string_record_type_8;
      break;
   case 32;
      return $string_record_type_32;
      break;
   default:
      return $string_record_type_0;
      }
   }

// \n in <br> umwandeln (EPG-Text etc.)
function newline2br($text) {
   $text = str_replace("\n", "<br><br>", $text);
   return $text;
   }

// Sender-Liste einlesen
if (file_exists($station_file)) {
   $station_list = file($station_file);
   foreach ($station_list as $line_num => $line) {
      // $station_array[10106] ist z.B. der Name des Senders #10106
      $station_array[trim(substr($line, strrpos($line, "0 ") + 2, -1))]=trim(substr($line, 0, strpos($line, " 0")));
      }
   } else {
      echo "Fehler: Kann Senderlisten-Datei ".$station_file." nicht einlesen!";
   }

// Array mit Nummern-Index umdrehen, Zuordnung "Key -> Daten" behalten
function reverse_numkey_array($item) {
   // Von hinten nach vorne lesen und hniten an neues Array anfuegen
   if ( !is_array($item) ) {
      echo "Fehler: \$item ist kein array! (Funktion 'reverse_numkey_array')";
   } else {
      end($item);
      do {
        $part1=key($item);
        $part2=current($item);
        $item_reverse[$part1]=$part2;
        } while(prev($item));
      return $item_reverse;
      }
   }

// Daten zur Box aus MAC-Adresse im Aufnahme-Dateinamen holen
function get_box_data($file) {
   global $receiver_data;
   // Erste 12 Zeichen des Dateinamens = MAC der Box
   $file = substr($file, 0, 12);
   // Welche Box hat diese MAC-Adresse?
   foreach ( $receiver_data as $box => $test ) {
      // War's Box mit ID $box?
      if ( $file == convert_mac($receiver_data[$box]['mac']) ) {
         // Daten von Box mit ID $box zurueckliefern
         return $receiver_data[$box];
         }
      }
   }

// Hilfsfunktion zum Sortieren der .crid-Dateien (Anzeigereihenfolge)
function cmp($a, $b) {
   // Dateinamen: MMMMMMMMMMMM_TTTTTTTTTTSSSSS.crid
   // Sortieren nach TTTTTTTTTT (Aufnahmezeit), also 15Z ab Z14
   global $crid_filelist_array;
   $cmp_a = substr($crid_filelist_array[$a], 13, 15);
   $cmp_b = substr($crid_filelist_array[$b], 13, 15);
   return strcmp($cmp_a, $cmp_b);
   }

// .crid-Dateien im Verzeichnis suchen
if ($handle = opendir($dir_video)) {
   $crid_filelist = "";
   // Verzeichnis durchlaufen
      while (false !== ($file = readdir($handle))) {
       // nur .crid-Dateien
       if (strstr($file,".crid")) {
          // Datei merken
          $crid_filelist .= $file." ";
          }
       }
   // Handle freigeben
   closedir($handle);
   } else {
      echo "Fehler: Kann Verzeichnis ".$dir_video." nicht einlesen!";
      exit(1);
   }

// Letztes Leerzeichen abschneiden, Array der Dateien aufbauen
$crid_filelist = trim($crid_filelist);
$crid_filelist_array = explode(" ", $crid_filelist);

// Sortieren nach Teil des Dateinamens (Funktion "cmp" oben)
uksort($crid_filelist_array, "cmp");

// Wie viele Aufnahmen?
$record_counter = count($crid_filelist_array);

// Array umdrehen (Neuste Aufnahmen zuerst)
$crid_filelist_array = reverse_numkey_array($crid_filelist_array);

// Zeitzone fuer Datums-Anzeige auf lokal setzen
date_default_timezone_set($my_timezone);

// Start HTML-Ausgabe
echo "<head>\n";
echo "<title>Aufnahmen auf dem Server ".$_SERVER['SERVER_NAME']." - ".$record_counter;
echo " Aufnahmen - ca. ".round(($dir_video_free_bytes/1024/1024/1024))." GB frei</title>\n";
?>
<style type="text/css">
<!--
a       { color: <? echo $text_color_link ?>; text-decoration: none; }
a:link  { color: <? echo $text_color_bright ?>; text-decoration: none; }
a:hover { color: <? echo $text_color_bright ?>; text-decoration: none; }

.hidden   { display: none; }
.unhidden { display: block; }
//-->
</style>
<script type="text/javascript">
function unhide(divID) {
   var item = document.getElementById(divID);
   if (item) {
      item.className=(item.className=='hidden')? 'unhidden':'hidden';
      }
   }
</script>
<!--[if lt IE 7]>
<script defer type="text/javascript" src="./pngfix.js"></script>
<![endif]-->
<meta http-equiv="Content-Type" content="text/html; charset=ISO-8859-1">
<meta name="description" content="Aufnahmen auf <?= $_SERVER['SERVER_NAME'] ?>">
<meta name="author" content="m740.php">
<meta name="keywords" content="siemens, m740av, m750s recordings, aufnahmen, php, script, skript">
<link rel="shortcut icon" type="image/x-icon" href="./favicon.ico" />
<link rel="stylesheet" href="/gui.css" type="text/css" media="screen" />
<?
echo "</head>\n";

// HTML Body
echo "<body bgcolor=\"".$text_color_background."\" ";
echo "text=\"".$text_color_normal."\" ";
echo "link=\"".$text_color_link."\" ";
echo "vlink=\"".$text_color_vlink."\" ";
echo "alink=\"".$text_color_alink."\" ";
echo "leftmargin=\"0\" topmargin=\"0\" marginwidth=\"0\" marginheight=\"0\">\n";
echo "<html>\n";
echo "<center><br>\n\n";

// Kopfzeile
echo "<!-- Box-Infos -->\n";
echo "<table width=\"".$table_width."\" order=\"0\" cellpadding=\"6\" cellspacing=\"0\">\n";
echo "<tr> ";
echo "<td colspan=\"2\" valign=\"top\" class=\"listtopic\">Speicherplatz auf Server \"".$_SERVER['SERVER_NAME']."\"</td>";
echo "</tr>";
echo "<tr>\n";
echo "<td align=\"left\" valign=\"top\" class=\"vncell\">";
echo "Festplatte belegt </td>";
echo "<td align=\"left\" valign=\"top\" class=\"vtable\">";
?>
<img style="vertical-align:middle" height="15" src="images/bar_left.gif"><img style="vertical-align:middle" height="15" width="<? echo (5 * $rec_pcnt) ?>" src="images/bar_blue.gif"><img style="vertical-align:middle" height="15" width="<? echo (500 - 5*$rec_pcnt) ?>" src="images/bar_gray.gif"><img style="vertical-align:middle" height="15" src="images/bar_right.gif">
<?
echo $rec_pcnt."%</td>";
echo "</tr>\n<tr>";

echo "<td align=\"left\" valign=\"top\" class=\"vncell\">";
echo "Systemstatus</td>";
echo "<td align=\"left\" valign=\"top\" class=\"vtable\">";
echo "<a href=\"/info.php\"><em>Hier klicken</em></a>";
echo "</td>\n";
echo "</tr>\n";

echo "</table>\n";
echo "<br>\n\n";

// Tabelle Kopf
echo "<!-- Haupttabelle -->\n";
echo "<table width=\"".$table_width."\" order=\"0\" cellpadding=\"6\" cellspacing=\"0\">\n";

// Zeile 0 - Ueberschriften
echo " <tr>\n";
echo "  <td colspan=\"3\" align=\"left\" valign=\"top\" class=\"listtopic\">Sendung &amp; EPG-Infotext, Aufnahmezeit, Details</b>\n";
echo " </tr>\n";

// Tabellen-Zeile 1
$table_row_counter = 1;

// Alle .crid-Dateien (Aufnahmen) durchgehen
foreach($crid_filelist_array as $file) {
   // Aufnahme zum Lesen oeffnen
   $timer = fopen($dir_video.$file, "rb");

   // Receiverdaten zur Aufnahme suchen (aus MAC-Adresse im Dateinamen)
   $recording_box = get_box_data($file);

   // Daten in Variablen schreiben
   $recording_box_host = $recording_box['host'];
   $recording_box_ip   = $recording_box['ip'];
   $recording_box_mac  = $recording_box['mac'];
   $recording_box_name = $recording_box['name'];

   // 4 Byte Version
   $vers = hexdec(bin2hex(fread($timer, 4)));

   // 5 Zeichen Sender-ID
   $senderID = substr($file,-10,5);
   $sendername =
   (isset($station_array[$senderID])
      ?$station_array[$senderID]['name']
      :NULL);

   // 8 Byte CID
   $cid = hexdec(bin2hex(fread($timer, 8)));

   // 4 Byte Status
   $status = hexdec(bin2hex(fread($timer, 4)));

   // 4 Byte Startzeit (UNIX Timestamp)
   $start = hexdec(bin2hex(fread($timer, 4)));

   // 4 Byte Endzeit (UNIX Timestamp)
   $ende = hexdec(bin2hex(fread($timer, 4)));

   // 4 Byte Unbekannt (3 mal)
   $unb1 = hexdec(bin2hex(fread($timer, 4)));
   $unb2 = hexdec(bin2hex(fread($timer, 4)));
   $unb3 = hexdec(bin2hex(fread($timer, 4)));

   // 4 Byte Aufnahme-Typ
   $epgf = hexdec(bin2hex(fread($timer, 4)));
   switch($epgf) {
      case 1;
         $recordtype = 1;
         break;
      case 2;
         $recordtype = 2;
         break;
      case 4;
         $recordtype = 4;
         break;
      case 8;
         $recordtype = 8;
         break;
      case 32;
         $recordtype = 32;
         break;
       default:
         $recordtype = $epgf;
      }

   // 4 Byte Serien-ID
   $serienid = hexdec(bin2hex(fread($timer, 4)));

   // 2 Byte Sperrflag
   $sperf = hexdec(bin2hex(fread($timer, 2)));

   // 4 Byte Laenge des Titels
   $length = hexdec(bin2hex(fread($timer, 4)));
   if ($length > 10000)
      $titel = ">>> Falsche Laengenangabe (Titel): " . $length;
   elseif ($length > 0)
   // N Byte Titel
      $titel = fread($timer, $length);
      $titel = clean_str($titel);

   // 4 Byte Anzahl Teile
   $parts =  hexdec(bin2hex(fread($timer, 4)));
   for ($i=1; $i<=$parts; $i++) {
      // Dateinamenpraefix
      $length = hexdec(bin2hex(fread($timer, 4)));
      if ($length > 10000)
         $filep = ">>> Falsche Laengenangabe (Dateinamenspraefix): " . $length;
      elseif ($length > 0)
         $filep = fread($timer, $length);

   // 4 Byte Tatsaechlicher Start (UNIX Timestamp)
   $rstart = hexdec(bin2hex(fread($timer, 4)));

   // 4 Byte Unbekannt (4 mal)
   $unb5 = hexdec(bin2hex(fread($timer, 4)));
   $unb6 = hexdec(bin2hex(fread($timer, 4)));
   $unb7 = hexdec(bin2hex(fread($timer, 4)));
   $unb8 = hexdec(bin2hex(fread($timer, 4)));

   // Array mit Aufnahmeteilen
   $recordparts[$i]=array(
       'filep'  => $filep
      ,'rstart' => $rstart
      ,'unb5'   => $unb5
      ,'unb6'   => $unb6
      ,'unb7'   => $unb7
      ,'unb8'   => $unb8
      );
      }

   // 4 Byte Folge, Episode
   $length = hexdec(bin2hex(fread($timer, 4)));
      if ($length > 10000)
         $episode = ">>> Falsche Laengenangabe (Episode): " . $length;
      elseif ($length > 0)
         $episode = fread($timer, $length);

   // Folge
   $episode = clean_str($episode);
   if($episode == $titel)
      $episode = "&nbsp;";

   // EPG-Text leeren, falls von letzter Aufnahme noch gesetzt
   $epg = "";

   // 4 Byte EPG-Text
   $length = hexdec(bin2hex(fread($timer, 4)));
   if ($length > 10000)
      $epg = ">>> Falsche Laengenangabe (Text): " . $length;
   elseif ($length > 0)
      $epg = fread($timer, $length);

   // EPG-Text saeubern
   $epg = clean_str($epg);
   if ($epg == "") {
      $epg = $no_epg_text;
      }

   // Array mit infos zur .crid-Datei aufbauen
   $cridDaten[$file] = array(
       'file'        => $file
      ,'version'     => $vers
      ,'sendername'  => $sendername
      ,'cid'         => $cid
      ,'status'      => $status
      ,'start'       => $start
      ,'ende'        => $ende
      ,'unb1'        => $unb1
      ,'unb2'        => $unb2
      ,'unb3'        => $unb3
      ,'epgf'        => $epgf
      ,'recordtype'  => $recordtype
      ,'serienid'    => $serienid
      ,'sperf'       => $sperf
      ,'titel'       => $titel
      ,'recordparts' => $recordparts
      ,'episode'     => $episode
      ,'epg'         => $epg
      );

   // Zeitzone fuer aktuelles Datum auf lokal setzen
   date_default_timezone_set($my_timezone);

   // Start und Ende laut EPG 
   $start_datum = date("d.m.Y",$cridDaten[$file]['start']);
   $start_zeit  = date("H:i",$cridDaten[$file]['start']);
   $ende_datum  = date("d.m.Y",$cridDaten[$file]['ende']);
   $ende_zeit   = date("H:i",$cridDaten[$file]['ende']);

   // Echter Start der Aufnahme (aus Datei)
   $start_datum_real = date("d.m.Y",$cridDaten[$file]['recordparts'][1]['rstart']);
   $start_zeit_real =  date("H:i",$cridDaten[$file]['recordparts'][1]['rstart']);

   // Zeitzone zum Berechnen der absoluten Dauer auf GMT (+0) setzen
   date_default_timezone_set('GMT');
   $aufnahme_dauer =  date("G\h ",($cridDaten[$file]['ende']-$cridDaten[$file]['start']));
   $aufnahme_dauer .= date("i\m\i\\n",($cridDaten[$file]['ende']-$cridDaten[$file]['start']));

   // Zeitzone auf lokal zuruecksetzen
   date_default_timezone_set($my_timezone);

   // Tabellen-Reihe
   echo "<!-- Aufnahme #".$divcounter." -->\n";
   echo " <tr>";

   // Spalte 1 - Titel
   echo "  <td align=\"left\" valign=\"top\" class=\"vncell\">";
   echo "<img style=\"float: right;\" src=\"".$dir_logos.$senderID.".png\" title=\"".$station_array[$senderID]."\" alt=\"".$station_array[$senderID]."\">";
   echo "<span class=\"rectitle\">".$cridDaten[$file]['titel']."</span><br>";
   echo "<blockquote>\n";
   echo newline2br($cridDaten[$file]['epg']);
   echo "</blockquote>\n";
   echo "</td>\n";

   // Spalte 2 - Sendezeit
   echo "  <td align=\"left\" valign=\"top\" class=\"vtable\">";
   echo "<nobr>";
   echo "Dauer: <b>".$aufnahme_dauer."</b>";
   echo "<br><br>";
   echo "<nobr>Sendezeit:<br><b>";
   echo $start_datum.", ".$start_zeit."</b><br>bis <b>";
   echo $ende_zeit."</b><br><br>";
   echo "<nobr>Aufnahmestart:<br><b>";
   echo $start_datum_real.", ".$start_zeit_real."</b></nobr>";
   echo "</td>\n";

   // Spalte 3 - Details
   echo "  <td align=\"left\" valign=\"top\" class=\"vtable\">";
   echo "<nobr>";
   echo "Sender: <b>".$station_array[$senderID]."</b><br>";
   echo "<br>";
   echo sperr_nr_to_text($cridDaten[$file]['sperf'])."<br>";
   echo status_nr_to_text($cridDaten[$file]['status'])."<br>";
   echo epgf_nr_to_text($cridDaten[$file]['epgf'])."<br>";
   $record_info = record_file_size($dir_video.$cridDaten[$file]['recordparts'][1]['filep']);
   echo "Einzeldateien: <b>".$record_info[1]."</b><br>";
   echo "Gr&ouml;&szlig;e: <b>".round($record_info[0]/1024/1024, 2)." MB</b><br>";
   echo "Quelle: <b>".$recording_box_name."</b><br>";
   echo "</td>\n";
   echo " </tr>\n";
   echo "<!-- Ende Aufnahme #".$divcounter." -->\n";

   // Zeilenzaehler erhoehen
   $table__row_counter = $table_row_counter + 1;

   // Zeahler fuer div erhoehen
   $divcounter = $divcounter + 1;
}

// Ende der Tabelle
echo "</table>\n";
echo "<!-- Ende Haupttabelle -->\n";
echo "</html>\n";
echo "<br>\n";
?>
